-- This module implements Infty type constructor. It takes a numeric type
-- and augments it with infinity and negative infinity; it implements 
-- plus, minus, times etc operations. Type FInt is also defined. It is an
-- Int type augmented with infinity and negative infinity.

module Infty where

data Infty a = F a | Infty | NegInfty
  deriving (Eq)

instance Ord a => Ord (Infty a) where
  compare (F x) (F y) = compare x y
  compare Infty Infty = EQ
  compare NegInfty NegInfty = EQ
  compare Infty _ = GT
  compare NegInfty _ = LT
  compare _ Infty = LT
  compare _ NegInfty = GT

instance Show a => Show (Infty a) where
  show (F x) = show x
  show Infty = "#"
  show NegInfty = "-#"

instance (Ord a,Num a) => Num (Infty a) where
  (+) (F x) (F y) = F (x+y)
  (+) Infty NegInfty = undefined
  (+) NegInfty Infty = undefined
  (+) Infty _ = Infty
  (+) _ Infty = Infty
  (+) NegInfty _ = NegInfty
  (+) _ NegInfty = NegInfty

  (*) (F x) (F y) = F (x*y)
  (*) Infty (F x) | x > 0 = Infty
  (*) Infty (F x) | x < 0 = NegInfty
  (*) Infty (F x) = undefined
  (*) NegInfty x = negate (Infty * x)

  negate (F x) = F (negate x)
  negate Infty = NegInfty
  negate NegInfty = Infty

  abs (F x) = F (abs x)
  abs _ = Infty

  signum (F x) = F (signum x)
  signum Infty = fromInteger 1
  signum NegInfty = negate (fromInteger 1)

  fromInteger x = F (fromInteger x)

type FInt = Infty Int
type FFloat = Infty Float